import re
from collections import Counter

chem_formula = re.compile('([A-Z][a-z]?)([-]?[0-9]*)')


def chem2atom(chem):
    """Transform a string with chemical formula to a dictionary with atom counts.

    Args:
        chem (str): A string with a chemical formula, or a chemical diff, e.g. 'C10HNa', 'C-1H-2O2'.
    Return:
        dict: counts of atoms.
    """
    global chem_formula
    return Counter({el: 1 if cnt is '' else int(cnt)
    				for el, cnt in chem_formula.findall(chem)})

def test_chem2atom():
	assert chem2atom('CH2Cu-1') == {'C':1,'H':2,'Cu':-1}

def add_formulas(f, g, check_positivity=True):
	res = Counter()
	for a in set(f.keys()) | set(g.keys()):
		cnt = f.get(a, 0) + g.get(a, 0)
		assert cnt >= 0, "Adding formulas resulted in a negative atom count. This might explain the dark matter!"
		if cnt:
			res[a] = cnt
	return res

def test_add_formulas():
	assert add_formulas({'C':10, 'H':20}, {'C':2, 'H':2}) == {'H':22, 'C':12}
	assert add_formulas({'C':10, 'H':20}, {'C':-10, 'H':-18}) == {'H':2}


import re
from collections import Counter

ptms = {
    "phosphorylation":              {"P":1, "H":1,  "O":3},
    "acetylation":                  {"C":2, "H":2,  "O":1},
    "glycation_by_hexose":          {"C":6, "H":10, "O":5},
    "amidation":                    {"H":1, "N":1,  "O":-1},
    "hydroxylation":                {"O":1},
    "methylation":                  {"C":1, "H":2},
    "ubiquitylation":               {"C":378, "H":627, "N":105, "O":117, "S":1},
    "pyroglutamic_acid":            {"H":-2, "O":-1},
    "sulfation":                    {"S":1, "O":3},
    "gamma_carboxyglutamic_acid":   {"C":1, "O":2},
    "dehydralation":                {"H":-2, "O": -1},
    "oxidation":                    {"O":1},
    "carboxymethylation":           {"C":2, "H":2, "O":2},
    "palmitic_acylation":           {"C":16,"H":30,"O":1},
    "oleic_acylation":              {"C":18,"H":32,"O":1},
    "arachidonic_acylation":        {"C":20,"H":30,"O":1},
    "docosahexanoic_acylation":     {"C":22,"H":30,"O":1},
    "carbamidomethylation":         {"C":2, "H":3, "N":1, "O":1}
}

typical_ptm_sites = {
    "phosphorylation": ("S", "T", "Y")
}

plgs_mod_pattern = re.compile("(\w+)\+(\w)\((\d+)\)")
# this should match: Oxidation+M(24), Carbamidomethyl+C(7), ...

plgs_notation = {
    'Carbamidomethyl':'carbamidomethylation',
    'Oxidation':'oxidation'
}
# this should be prolonged

def compare_seq_and_mod(seq, mod):
    assert all(seq[int(i)-1] == aa for _, aa, i in plgs_mod_pattern.findall(mod))


def test_parsing():
    """This tests that Stefan is right about +C(17) meaning that the 17th amino acid is C."""
    for seq, mod in zip(sm.sequence, sm.modification):
        compare_seq_and_mod(seq, mod)


def PLGSptms2atom(mod, seq=''):
    """For a given PLGS modification, calculate the atom_diff.

    Args:
        mod (str): A string with PLGS modification (see 'plgs_notation').
        seq (str): The amino acidic sequence that must be checked.
    Returns:
        collections.Counter: An atom_diff.
    """
    global plgs_mod_pattern, plgs_notation
    atom_diff = Counter()
    for name, aa, i in plgs_mod_pattern.findall(mod):
        atom_diff.update(ptms[plgs_notation[name]])
        if seq:
            assert seq[int(i)-1] == aa, "The modification is poorly specified!"
    return atom_diff



from aa2atom import aa2atom
import re

aaseq = "AAPAPAAPGGL"
aa2atom(aaseq)
aa2atom(aaseq, no_water=)

A1 = aa2atom("AAPAPAAPGGL")
A2 = aa2atom("AASSAPAAPGGL")

A1 + A2

W = Counter("AAPAPAAPGGL")

# check these calculations
# get rid of the LC dependency

